"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformMainMenu = exports.splitActions = exports.simpleTypes = exports.setObjectWithPath = exports.serverFormatPostfix = exports.resolveRelativeURL = exports.resolveElixirDate = exports.leftToRightLanguages = exports.keyboard_shortcuts = exports.isValidStaticAction = exports.isValidMemberAction = exports.isValidAction = exports.isSuccessResponse = exports.isSimpleType = exports.isSimpleGroup = exports.isRangeOperator = exports.isNotUndefined = exports.isMultiOperator = exports.isImageFileType = exports.isHTML = exports.isFileType = exports.isEqual = exports.isEmpty = exports.isAvailableAction = exports.getTdlWithPath = exports.getTdlKeys = exports.getObjectWithPath = exports.getObjectUsingType = exports.getObjectUsingTdlKeys = exports.getObject = exports.getDateFormat = exports.getClientPKS = exports.getActionKey = exports.findObjInCollection = exports.evalStyle = exports.dynamic = exports.copyObj1ToObj2 = exports.cloneDeep = exports.base64Encode = exports.allOperators = exports.Simple = exports.HtmlRegex = exports.Composition = exports.Collection = exports.Association = void 0;
var _lodash = _interopRequireDefault(require("lodash"));
var _reactFastCompare = _interopRequireDefault(require("react-fast-compare"));
var _moment = _interopRequireDefault(require("moment"));
var _axios = require("../config/axios");
var _shortcutHandlers = require("./shortcutHandlers");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const Composition = exports.Composition = "Composition";
const Collection = exports.Collection = "Collection";
const Association = exports.Association = "Association";
const serverFormatPostfix = exports.serverFormatPostfix = "_$string";
const Simple = exports.Simple = "Simple";
const allOperators = exports.allOperators = {
  single: ["=", "!=", ">", ">=", "<", "<=", "!"],
  range: ["Between", "<>"],
  multi: ["In", "in", "LikeIn"]
};
const leftToRightLanguages = exports.leftToRightLanguages = ["English"];
const simpleTypes = exports.simpleTypes = ["BlobText", "String", "Double", "Long", "EMAIL", "TypeName", "Password", "Boolean", "Date", "Time", "DateTime", "Duration", "Text", "Double16", "MultiString"];
const DATE_FORMATS = {
  Time: "HH:mm",
  Duration: "HH:mm:ss",
  Date: "DD/MM/YYYY",
  DateTime: "YYYY-MM-DD HH:mm:ss"
};
const dynamic = exports.dynamic = "dynamic";
const evalStyle = (customStyleObject, value, meta, evaluatedStyle) => {
  for (let key in customStyleObject) {
    if (typeof customStyleObject[key] == "function") {
      evaluatedStyle[key] = customStyleObject[key](value, meta);
    } else if (typeof customStyleObject[key] == "object") {
      evaluatedStyle[key] = {};
      evalStyle(customStyleObject[key], value, meta, evaluatedStyle[key]);
    } else {
      evaluatedStyle[key] = customStyleObject[key];
    }
  }
};
exports.evalStyle = evalStyle;
const keyboard_shortcuts = exports.keyboard_shortcuts = [{
  modifiers: ["ctrlKey", "shiftKey"],
  keys: ["1", "2", "3", "4", "5", "6", "7", "8", "9"],
  handler: (physicalKey, viewStatus, dispatch) => (0, _shortcutHandlers.onFileShortcut)(physicalKey, viewStatus, dispatch)
}, {
  modifiers: ["ctrlKey"],
  keys: ["1", "2", "3", "4", "5", "6", "7", "8", "9"],
  handler: (physicalKey, viewStatus, dispatch) => (0, _shortcutHandlers.onActionShortcut)(physicalKey, viewStatus, dispatch)
}, {
  modifiers: ["ctrlKey"],
  keys: ["S"],
  handler: (physicalKey, viewStatus, dispatch) => (0, _shortcutHandlers.onSearchShortcut)(viewStatus, dispatch)
}];
const isMultiOperator = op => op && (op.includes(",") || op.includes(";"));
exports.isMultiOperator = isMultiOperator;
const isRangeOperator = op => op && (op === "[]" || op === "][");
exports.isRangeOperator = isRangeOperator;
const isEqual = (object1, object2) => (0, _reactFastCompare.default)(object1, object2);
exports.isEqual = isEqual;
const isEmpty = object => _lodash.default.isEmpty(object);
exports.isEmpty = isEmpty;
const isSimpleGroup = _ref => {
  let {
    masterKey
  } = _ref;
  return masterKey.includes("*");
};
exports.isSimpleGroup = isSimpleGroup;
const cloneDeep = object => _lodash.default.cloneDeep(object);
exports.cloneDeep = cloneDeep;
const getDateFormat = type => DATE_FORMATS[type];
exports.getDateFormat = getDateFormat;
const resolveRelativeURL = url => "".concat(_axios.baseServerURL).concat(url.startsWith("/") ? url : "/".concat(url));
exports.resolveRelativeURL = resolveRelativeURL;
const isFileType = type => type === "com.elixir.data.File";
exports.isFileType = isFileType;
const isImageFileType = type => type === "com.elixir.data.ImageFile";
exports.isImageFileType = isImageFileType;
const isSuccessResponse = response => response.status === 200 && response.data && response.data.output && response.data.output.errorMessages.length === 0;
exports.isSuccessResponse = isSuccessResponse;
const addExtraPath = (step, obj, currentPath, order, isExtraPath, i) => {
  const {
    basedOn,
    tdlId,
    stepLabel
  } = step;
  let paths = currentPath.split('/');
  let basedOnOrder = order.split('/');
  let itemOrder = basedOnOrder[i];
  let itemPath = paths[i];
  if (i === paths.length - 1) {
    if (!obj[itemPath]) {
      obj[itemPath] = {
        label: itemPath,
        basedOn: {
          ...basedOn,
          order: itemOrder
        },
        items: [],
        processes: {}
      };
    } else if (!isExtraPath && obj[itemPath]) {
      obj[itemPath] = {
        ...obj[itemPath],
        basedOn: {
          ...basedOn,
          order: itemOrder
        }
      };
    }
    obj[itemPath].items.push({
      label: stepLabel,
      basedOn,
      tdlId,
      order: basedOnOrder[i + 1]
    });
  } else {
    if (!obj[itemPath]) {
      obj[itemPath] = {
        label: itemPath,
        basedOn: {
          ...basedOn,
          order: itemOrder
        },
        items: [],
        processes: {}
      };
    } else if (!isExtraPath && obj[itemPath]) {
      obj[itemPath] = {
        ...obj[itemPath],
        basedOn: {
          ...basedOn,
          order: itemOrder
        }
      };
    }
    addExtraPath(step, obj[itemPath].processes, currentPath, order, isExtraPath, i + 1);
  }
};
const transformMainMenu = object => {
  const menuObject = {};
  let menuArray = [];
  let pathArray = [];
  object.map(item => {
    const {
      basedOnModuleLabel,
      basedOnProcessLabel,
      basedOn,
      tdlId,
      stepLabel,
      order
    } = item;
    if (item.evaluatedPaths) {
      const pathsObject = JSON.parse(item.evaluatedPaths);
      const pathsObjectEntries = Object.entries(pathsObject);
      pathArray = pathsObjectEntries[0][1];
      pathArray.forEach(path => {
        addExtraPath(item, menuObject, path.path, path.order, true, 0);
      });
    } else {
      addExtraPath(item, menuObject, basedOnModuleLabel + '/' + basedOnProcessLabel, basedOn.moduleOrder + "/" + basedOn.prcssOrder + "/" + order, false, 0);
    }
  });
  menuArray = convertProcesToArray(menuObject);
  menuArray.map(process => {
    sortObject(process);
  });
  menuArray.sort((x1, x2) => x1.basedOn.order - x2.basedOn.order);
  return menuArray;
};
exports.transformMainMenu = transformMainMenu;
const convertProcesToArray = obj => {
  let a = [];
  Object.keys(obj).map((key, value) => {
    let plist = undefined;
    if (obj[key].processes) {
      plist = convertProcesToArray(obj[key].processes);
    }
    a.push({
      ...obj[key],
      processes: plist
    });
  });
  return a;
};
const sortObject = obj => {
  if (obj.processes && obj.processes.length > 0) {
    obj.processes.sort((x1, x2) => x1.basedOn.order - x2.basedOn.order);
    obj.processes.map(process => {
      sortObject(process);
    });
  }
  if (obj.items && obj.items.length > 0) {
    obj.items.sort((x1, x2) => x1.order - x2.order);
  }
};
const isValidAction = _ref2 => {
  let {
    hasPermission
  } = _ref2;
  return hasPermission === "true";
};
exports.isValidAction = isValidAction;
const isAvailableAction = _ref3 => {
  let {
    isAvailable
  } = _ref3;
  return isAvailable === "true";
};
exports.isAvailableAction = isAvailableAction;
const isValidMemberAction = _ref4 => {
  let {
    isMember
  } = _ref4;
  return isMember === "true";
};
exports.isValidMemberAction = isValidMemberAction;
const isValidStaticAction = _ref5 => {
  let {
    isMember,
    isViewAction
  } = _ref5;
  return isMember === "false" || isMember === "true" && isViewAction === "false";
};
exports.isValidStaticAction = isValidStaticAction;
const getActionKey = _ref6 => {
  let {
    tdlId,
    actionLabel
  } = _ref6;
  return "".concat(tdlId).concat(actionLabel);
};
exports.getActionKey = getActionKey;
const splitActions = (actions, splitNumber) => {
  const splittedActions = [[], []];
  splittedActions[0] = actions.slice(0, Math.min(splitNumber, actions.length));
  if (actions.length > splitNumber) {
    splittedActions[1] = actions.slice(splitNumber, actions.length);
  }
  return splittedActions;
};
exports.splitActions = splitActions;
const isSimpleType = type => !!simpleTypes.find(item => item === type);
exports.isSimpleType = isSimpleType;
const HtmlRegex = exports.HtmlRegex = new RegExp(/<\/?\w+((\s+\w+(\s*=\s*(?:".*?"|'.*?'|[\^'">\s]+))?)+\s*|\s*)?>/);
const isHTML = str => str && typeof str === "string" && (str.includes("<elixirhtml") || HtmlRegex.test(str));
exports.isHTML = isHTML;
const resolveElixirDate = (value, type, timeZoneOffset) => {
  const format = DATE_FORMATS[type];
  if (value) {
    //get String
    if (Number.isNaN(Number(value))) {
      let val = value;
      if (value.startsWith('"')) {
        val = value.substring(1, value.length - 1);
      }
      return (0, _moment.default)(val, format);
    }
    //get Long
    const dateObject = new Date(Number(value));
    //server timezone
    if (type === "Date" || type === "DateTime" || type === "Time") dateObject.setMinutes(dateObject.getMinutes() - -dateObject.getTimezoneOffset() + timeZoneOffset / 60000);
    if (type === "Duration") dateObject.setMinutes(dateObject.getMinutes() - -dateObject.getTimezoneOffset());
    return (0, _moment.default)(dateObject);
  }
  return "";
};
exports.resolveElixirDate = resolveElixirDate;
const getObject = (path, object, type, tdlKeys) => {
  if (path === undefined || path === "") {
    return object;
  }
  const arr = path.split(".");
  const curr = arr[0];
  if (curr !== "") {
    if (curr === "*") {
      arr.splice(0, 1);
      const newPath = arr.join(".");
      return getObject(newPath, object, type, tdlKeys);
    } else {
      if (curr.indexOf("[") !== -1) {
        const keyName = curr.split("[")[0];
        const index = parseInt(curr.split("[")[1].split("]")[0], 0);
        arr.splice(0, 1);
        const newPath = arr.join(".");
        if (Array.isArray(object)) {
          return getObject(newPath, object[index], type, tdlKeys);
        } else if (object[keyName][index] !== undefined) {
          return getObject(newPath, object[keyName][index], type, tdlKeys !== undefined ? tdlKeys["subKeys"][path.split(".")[0]] : tdlKeys);
        }
        return "";
      } else {
        if (object[curr] === undefined || object[curr] == null) {
          let currentType = "Simple";
          if (!curr.includes("_$string")) {
            currentType = tdlKeys === undefined ? type : tdlKeys["subKeys"][curr]["keyType"];
          }
          if (arr.length === 1) {
            if (currentType === Collection) {
              object[curr] = [];
            } else if (currentType === Composition) {
              object[curr] = {};
            } else {
              object[curr] = "";
            }
          } else {
            if (arr[1] === "*") {
              object = [];
            } else {
              object[curr] = {};
            }
          }
        }
        arr.splice(0, 1);
        const newPath = arr.join(".");
        return getObject(newPath, object[curr], type, tdlKeys !== undefined ? tdlKeys["subKeys"][path.split(".")[0]] : undefined);
      }
    }
  }
};
exports.getObject = getObject;
const getObjectWithPath = (object, path, type, tdlKeys) => {
  return getObject(path, object, type, tdlKeys);
};
exports.getObjectWithPath = getObjectWithPath;
const getObjectUsingType = (object, path, type) => {
  return getObject(path, object, type, undefined);
};
exports.getObjectUsingType = getObjectUsingType;
const getObjectUsingTdlKeys = (object, path, tdlKeys) => {
  return getObject(path, object, undefined, tdlKeys);
};
exports.getObjectUsingTdlKeys = getObjectUsingTdlKeys;
const getTdlKeys = (tdlKeys, path) => {
  if (!tdlKeys || !tdlKeys.subKeys) return tdlKeys;
  if (path === "") {
    return tdlKeys.subKeys;
  }
  const arr = path.split(".");
  let index = 0;
  let currentTdlKeys = tdlKeys.subKeys[arr[index]];
  index += 1;
  while (index < arr.length) {
    currentTdlKeys = tdlKeys.subKeys[arr[index]];
    index += 1;
  }
  return currentTdlKeys;
};
exports.getTdlKeys = getTdlKeys;
const findObjInCollection = (collection, pk) => {
  for (const object in collection) {
    if (collection[object].pk === pk) {
      return collection[object];
    }
  }
  return undefined;
};
exports.findObjInCollection = findObjInCollection;
const getClientPKS = collection => {
  const keys = [];
  for (const key in collection) {
    if (collection[key]) {
      keys.push(collection[key].pk);
    }
  }
  return keys;
};
exports.getClientPKS = getClientPKS;
const getTdlWithPath = (path, tdl) => {
  if (!tdl) {
    return null;
  }
  const arr = path.split(".");
  if (arr.length === 1 && path === "") {
    return tdl;
  }
  const curr = arr[0].split("-");
  arr.splice(0, 1);
  const newPath = arr.join(".");
  if (curr[1] !== undefined) {
    return getTdlWithPath(newPath, tdl[curr[0]][curr[1]]);
  }
  return getTdlWithPath(newPath, tdl[curr[0]]);
};
exports.getTdlWithPath = getTdlWithPath;
const setObjectWithPath = (path, object, value) => {
  const arr = path.split(".");
  if (arr.length === 1 || arr.length === 2 && arr[1] === "*") {
    if (path.indexOf("[") !== -1) {
      const keyName = path.split("[")[0];
      const index = path.split("[")[1].split("]")[0];
      if (Array.isArray(object)) {
        object[index] = value;
      } else {
        object[keyName][index] = value;
      }
    } else {
      if (arr[1] === "*") {
        object[arr[0]] = value;
      } else {
        // object[path] = Object.assign(value , object[path]);
        object[path] = value;
        if (value != {}) {
          object[path + serverFormatPostfix] = value && value.toString() || value;
        }
      }
    }
  } else {
    const curr = arr[0];
    arr.splice(0, 1);
    const newPath = arr.join(".");
    if (curr === "*") {
      setObjectWithPath(newPath, object, value);
    } else if (curr.indexOf("[") !== -1) {
      const keyName = curr.split("[")[0];
      const index = curr.split("[")[1].split("]")[0];
      if (Array.isArray(object)) {
        setObjectWithPath(newPath, object[index], value);
      } else {
        setObjectWithPath(newPath, object[keyName][index], value);
      }
    } else {
      setObjectWithPath(newPath, object[curr], value);
    }
  }
};
exports.setObjectWithPath = setObjectWithPath;
const base64Encode = data => {
  // Check if btoa is available (browsers and React Native)
  if (typeof window === 'object' && typeof window.btoa == 'function') {
    return window.btoa(data);
  } else if (typeof Buffer === 'function') {
    // Node.js
    return Buffer.from(data, 'binary').toString('base64');
  } else {
    throw new Error('Base64 encoding not supported in this environment.');
  }
};
exports.base64Encode = base64Encode;
const copyObj1ToObj2 = (obj1, obj2) => {
  _lodash.default.extend(obj2, obj1);
  return obj2;
};
exports.copyObj1ToObj2 = copyObj1ToObj2;
const isNotUndefined = (object, path) => {
  const arr = path.split(".");
  const curr = arr[0];
  let newObject = null;
  if (curr.indexOf("[") !== -1) {
    const keyName = curr.split("[")[0];
    const index = curr.split("[")[1].split("]")[0];
    if (object[keyName] === undefined || object[keyName][index] === undefined) {
      return false;
    }
    newObject = object[keyName][index];
  } else {
    if (object[curr] === undefined) {
      return false;
    }
    newObject = object[curr];
  }
  arr.splice(0, 1);
  const newPath = arr.join(".");
  if (newPath === "") {
    return true;
  }
  isNotUndefined(newObject, newPath);
};
exports.isNotUndefined = isNotUndefined;